(use-modules ((srfi srfi-1)  :select (every)))
(use-modules ((srfi srfi-13) :select (string-append/shared)))

(define (values->list values)
  (call-with-values (lambda () values) list))

(define (list-insert! list object)
  (set-cdr! list (cons object (cdr list))))

(define (vector-copy vec)
  (let* ((len (vector-length vec))
         (new (make-vector len #f)))
    (vector-move-left! vec 0 len new 0)
    new))

'(define (alist-satisfier criteria)
  (define (alist-satisfies? alist)
    (define (satisfied? criterion)
      (let ((pair (assq (car criterion) alist))
            (match-values (cdr criterion)))
        (if pair
          (or (eq? match-values 'any)
              (member (cdr pair) match-values))
          (eq? match-values 'none))))
    (every satisfied? criteria))
  alist-satisfies?)

(define (hash-for-each proc table)
  (hash-fold (lambda (key value so-far) (proc key value)) #f table))

'(define define-hash-entry!
  (let ()
    (define (stringify obj)
      (if (string? obj)
        obj
        (symbol->string obj)))
    (define (symbolify . args)
      (string->symbol (apply string-append/shared args)))
    (define (define-hash-entry! module typename key-fields tail-fields)
      (set! typename (stringify typename))
      (module-define! module (symbolify "make-" typename "-key")
                      (lambda args
                        (and (= (length args) (length key-fields))
                             (apply vector args))))
      (module-define! module (symbolify "make-" typename "-tail")
                      (lambda () (make-vector (length tail-fields) #f)))
      (for-each
       (lambda (fields pair-accessor make-setter?)
         (let loop ((i (1- (length fields))))
           (when (>= i 0)
             (let ((getter (lambda (obj) (vector-ref (pair-accessor obj) i))))
               (module-define!
                module
                (symbolify typename ":" (stringify (list-ref fields i)))
                (if make-setter?
                  (make-procedure-with-setter
                   getter
                   (lambda (obj value)
                     (vector-set! (pair-accessor obj) i value)))
                  getter)))
             (loop (1- i)))))
       (list key-fields tail-fields)
       (list car        cdr)
       (list #f         #t)))
    define-hash-entry!))
